@@ -61,19 +61,19 @@ MagiSolver::MagiSolver(const arma::mat & yFull,
         distSignedFull(tvecFull.size(), tvecFull.size()),
         indicatorRowWithObs(yFull.n_rows),
         indicatorMatWithObs(yFull.n_rows, yFull.n_cols, arma::fill::zeros),
-        phiAllDimensions(2, yFull.n_cols),
+        // phiAllDimensions(2, yFull.n_cols),
         llikxthetasigmaSamples(1 + yFull.size() + odeModel.thetaSize + sigmaSize, niterHmc, nEpoch)
 {
-    if(kernel != "generalMatern"){
-        throw std::runtime_error("only generalMatern kernel has full support");
-    }
-    
+    // if(kernel != "generalMatern"){
+    //     throw std::runtime_error("only generalMatern kernel has full support");
+    // }
+
     if (stepSizeFactorHmcInput.n_elem == 0){
       stepSizeFactorHmc = arma::ones(1);
     }else{
       stepSizeFactorHmc = stepSizeFactorHmcInput;
     }
-    
+
     // generate intermediate data
     if(useBand && useMean){
         loglikflag = "withmeanBand";
@@ -108,14 +108,19 @@ MagiSolver::MagiSolver(const arma::mat & yFull,
 
     if(kernel == "matern"){
         kernelCov = maternCov;
+        phiAllDimensions.set_size(2, yFull.n_cols);
     }else if(kernel == "rbf"){
         kernelCov = rbfCov;
+        phiAllDimensions.set_size(2, yFull.n_cols);
     }else if(kernel == "compact1"){
         kernelCov = compact1Cov;
+        phiAllDimensions.set_size(2, yFull.n_cols);
     }else if(kernel == "periodicMatern"){
         kernelCov = periodicMaternCov;
+        phiAllDimensions.set_size(3, yFull.n_cols);
     }else if(kernel == "generalMatern"){
         kernelCov = generalMaternCov;
+        phiAllDimensions.set_size(2, yFull.n_cols);
     }else{
         throw std::runtime_error("kernel is not specified correctly");
     }
@@ -130,11 +135,13 @@ void MagiSolver::setupPhiSigma() {
                                                 kernel,
                                                 -1,
                                                 useFrequencyBasedPrior);
-            phiAllDimensions = arma::reshape(phisig.subvec(0, 2*ydim - 1).eval(), 2, ydim);
-            sigmaInit = phisig.subvec(2*ydim, 2*ydim);
+            int phiDim = phisig.n_elem - 1;
+            phiAllDimensions = arma::reshape(phisig.subvec(0, phiDim*ydim - 1).eval(), phiDim, ydim);
+            sigmaInit = phisig.subvec(phiDim*ydim, phiDim*ydim);
         }else{
             sigmaInit.resize(ydim);
             arma::uvec sucess(ydim);
+            int phiDim;
             for(unsigned j = 0; j < ydim; j++){
                 if(idxColElemWithObs[j].size() >= 3){
                     const arma::vec & yObsCol = yFull.col(j).eval().elem(idxColElemWithObs[j]);
@@ -144,8 +151,9 @@ void MagiSolver::setupPhiSigma() {
                                                         kernel,
                                                         -1,
                                                         useFrequencyBasedPrior);
-                    phiAllDimensions.col(j) = phisig.subvec(0, 1);
-                    sigmaInit(j) = phisig(2);
+                    phiDim = phisig.n_elem - 1;
+                    phiAllDimensions.col(j) = phisig.subvec(0, phiDim - 1);
+                    sigmaInit(j) = phisig(phiDim);
                     sucess(j) = 1;
                 }else{
                     sucess(j) = 0;
@@ -157,8 +165,9 @@ void MagiSolver::setupPhiSigma() {
 
             const arma::uvec & failedDim = arma::find(sucess == 0);
             sigmaInit(failedDim).fill(sigmaMean);
-            phiAllDimensions.submat(arma::uvec({0}), failedDim).fill(phiMean(0));
-            phiAllDimensions.submat(arma::uvec({1}), failedDim).fill(phiMean(1));
+            for(unsigned j = 0; j < phiDim; j++){
+                phiAllDimensions.submat(arma::uvec({j}), failedDim).fill(phiMean(j));
+            }
         }
     }else if(!sigmaExogenous.empty() && phiExogenous.empty()){
         if(useScalerSigma){
@@ -167,11 +176,13 @@ void MagiSolver::setupPhiSigma() {
                                                 kernel,
                                                 sigmaExogenous(0),
                                                 useFrequencyBasedPrior);
-            phiAllDimensions = arma::reshape(phisig.subvec(0, 2*ydim - 1).eval(), 2, ydim);
+            int phiDim = phisig.n_elem;
+            phiAllDimensions = arma::reshape(phisig.subvec(0, phiDim*ydim - 1).eval(), phiDim, ydim);
             sigmaInit = sigmaExogenous.subvec(0, 0);
         }else{
             sigmaInit = sigmaExogenous;
             arma::uvec sucess(ydim);
+            int phiDim;
             for(unsigned j = 0; j < ydim; j++){
                 if(idxColElemWithObs[j].size() >= 3){
                     const arma::vec & yObsCol = yFull.col(j).eval().elem(idxColElemWithObs[j]);
@@ -181,7 +192,8 @@ void MagiSolver::setupPhiSigma() {
                                                         kernel,
                                                         sigmaExogenous(j),
                                                         useFrequencyBasedPrior);
-                    phiAllDimensions.col(j) = phisig.subvec(0, 1);
+                    phiDim = phisig.n_elem;
+                    phiAllDimensions.col(j) = phisig.subvec(0, phiDim - 1);
                     sucess(j) = 1;
                 }else{
                     sucess(j) = 0;
@@ -191,8 +203,9 @@ void MagiSolver::setupPhiSigma() {
             const arma::vec & phiMean = arma::mean(phiAllDimensions.cols(sucessDim), 1);
 
             const arma::uvec & failedDim = arma::find(sucess == 0);
-            phiAllDimensions.submat(arma::uvec({0}), failedDim).fill(phiMean(0));
-            phiAllDimensions.submat(arma::uvec({1}), failedDim).fill(phiMean(1));
+            for(unsigned j = 0; j < phiDim; j++){
+                phiAllDimensions.submat(arma::uvec({j}), failedDim).fill(phiMean(j));
+            }
         }
     }else if(!sigmaExogenous.empty() && !phiExogenous.empty()) {
         if(useScalerSigma){
@@ -315,6 +328,10 @@ void MagiSolver::initMissingComponent() {
         return;
     }
 
+    if (kernel == "periodicMatern" && phiExogenous.empty() && xInitExogenous.empty()) {
+        throw std::runtime_error("xInit and phi must be manually specified for unobserved components if using periodicMatern kernel,\n and set skipMissingComponentOptimization = TRUE");
+    }
+
     const arma::uvec & observedComponentDim = arma::find(nobsEachDim >= 0);
     if(xInitExogenous.empty()) {
         for (auto iPtr = missingComponentDim.begin(); iPtr < missingComponentDim.end(); iPtr++) {
